/*
 * Licensed Materials -- Property of IBM
 * 5724-L73
 * (C) Copyright International Business Machines Corp. 2004, 2005
 * All Rights Reserved.
 * U.S. Government Users Restricted Rights -- Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

package com.tivoli.am.fim.echo.service;

import java.io.ByteArrayInputStream;
import java.io.StringWriter;
import java.security.Principal;
import java.util.Iterator;
import java.util.Set;

import javax.security.auth.Subject;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;

import com.ibm.websphere.security.WSSecurityException;
import com.ibm.websphere.security.auth.WSSubject;

/**
 * <code>EchoServiceSoapBindingImpl</code> implements the echo web service.
 */
public class EchoServiceSoapBindingImpl implements com.tivoli.am.fim.echo.service.EchoService_PortType {
	
	/**
	 * Formats the XML string to make it easier to read.
	 * 
	 * @param xmlString the XML string
	 * @return the formatted string or, if the formatting failed, the original string
	 */
	private String formatXmlString(String xmlString) {
        try {
            if (xmlString == null || xmlString.length() == 0) {
                return xmlString;
            }
            
            DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
            documentBuilderFactory.setNamespaceAware(true);
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            Document document = documentBuilder.parse(new ByteArrayInputStream(xmlString.getBytes()));

            DOMSource domSource = new DOMSource(document);
            StringWriter stringWriter = new StringWriter();
            StreamResult streamResult = new StreamResult(stringWriter);
            
            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer = transformerFactory.newTransformer();
            transformer.setOutputProperty(OutputKeys.INDENT, "yes");
            transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "4");
            
            transformer.transform(domSource, streamResult);
            
            return stringWriter.toString();
        }
        
        catch (Exception exception) {
            // Can't parse or format so just return the original string
            return xmlString;
        }
    }

    /**
     * Displays the principals in a subject.
     * 
     * @param subject the subject
     * @param stringBuffer the string buffer into which the information is built
     */
    private void displayPrincipals(Subject subject, StringBuffer stringBuffer) {
		try {
			stringBuffer.append("\r\nPrincipals:\r\n");

			Set principals = subject.getPrincipals();
			
			if (principals == null || principals.isEmpty()) {
				stringBuffer.append("There are no principals in the subject.\r\n");
				
				return;				
			}
			
			Iterator iterator = principals.iterator();
			
			while (iterator.hasNext()) {
				Principal principal = (Principal) iterator.next();
				
				stringBuffer.append("Principal: ");
				stringBuffer.append(principal.getName());
				stringBuffer.append("\r\n");
			}			
		}
		
		catch (Exception exception) {
			stringBuffer.append("An error occurred attempting to display the principals in the subject.\r\n");
			stringBuffer.append(exception.getMessage());
			stringBuffer.append("\r\n");

			exception.printStackTrace();
		}
    }
    
    /**
     * Displays the public credentials in a subject.
     * 
     * @param subject the subject
     * @param stringBuffer the string buffer into which the information is built
     */
    private void displayPublicCredentials(Subject subject, StringBuffer stringBuffer) {
		try {
			stringBuffer.append("\r\nPublic Credentials:\r\n");

			Set publicCredentials = subject.getPublicCredentials();

			if (publicCredentials == null || publicCredentials.isEmpty()) {
				stringBuffer.append("There are no public credentials in the subject.\r\n");
				
				return;				
			}
			
			Iterator iterator = publicCredentials.iterator();
			
			while (iterator.hasNext()) {
				Object publicCredential = iterator.next();
				
				stringBuffer.append("Public credential type: ");
				stringBuffer.append(publicCredential.getClass().getName());
				stringBuffer.append("\r\n");
				
				stringBuffer.append("Public credential value: ");
				stringBuffer.append(publicCredential.toString());
				stringBuffer.append("\r\n");
			}
		}
		
		catch (Exception exception) {
			stringBuffer.append("An error occurred attempting to display the public credentials in the subject.\r\n");
			stringBuffer.append(exception.getMessage());
			stringBuffer.append("\r\n");

			exception.printStackTrace();
		}
    }
    
    /**
     * Displays the private credentials in a subject.
     * 
     * @param subject the subject
     * @param stringBuffer the string buffer into which the information is built
     */
    private void displayPrivateCredentials(Subject subject, StringBuffer stringBuffer) {
		try {
			stringBuffer.append("\r\nPrivate Credentials:\r\n");

			Set privateCredentials = subject.getPrivateCredentials();

			if (privateCredentials == null || privateCredentials.isEmpty()) {
				stringBuffer.append("There are no private credentials in the subject.\r\n");
				
				return;				
			}
			
			Iterator iterator = privateCredentials.iterator();
			
			while (iterator.hasNext()) {
				Object privateCredential = iterator.next();
				
				stringBuffer.append("Private credential type: ");
				stringBuffer.append(privateCredential.getClass().getName());
				stringBuffer.append("\r\n");
				
				stringBuffer.append("Private credential value: ");
				stringBuffer.append(privateCredential.toString());
				stringBuffer.append("\r\n");
			}
		}
		
		catch (Exception exception) {
			stringBuffer.append("An error occurred attempting to display the private credentials in the subject.\r\n");
			stringBuffer.append(exception.getMessage());
			stringBuffer.append("\r\n");

			exception.printStackTrace();
		}
    }
    
	/**
	 * Displays the logged in subject, if any.
	 * 
	 * NB. This application must be granted 
	 * javax.security.auth.AuthPermission("wssecurity.getCallerSubject"). 
	 * 
     * @param stringBuffer the string buffer into which the information is built
	 */
	private void displaySubject(StringBuffer stringBuffer) {
		try {
			Subject subject = WSSubject.getCallerSubject();
			
			if (subject == null) {
				stringBuffer.append("The web service has no JAAS subject.\r\n");
				
				return;				
			}
			
			stringBuffer.append("The web service JAAS subject contains:\r\n");

			displayPrincipals(subject, stringBuffer);
			displayPublicCredentials(subject, stringBuffer);
			displayPrivateCredentials(subject, stringBuffer);
		}
		
		catch (WSSecurityException exception) {
			stringBuffer.append("An error occurred attempting to display the subject.\r\n");
			stringBuffer.append(exception.getMessage());
			stringBuffer.append("\r\n");

			exception.printStackTrace();
		}
	}
	
	/**
	 * Displays the logged in subject, if any, to stdout.
	 * 
	 * NB. This application must be granted 
	 * javax.security.auth.AuthPermission("wssecurity.getCallerSubject"). 
	 * 
	 * @param stringBuffer the string buffer into which the information is built
	 */
	private void displaySubject() {
		StringBuffer stringBuffer = new StringBuffer();

		displaySubject(stringBuffer);
		System.out.print(stringBuffer.toString());
	}
	
	/**
	 * Echoes text.
	 * 
	 * @param text the text to echo
	 * @return the echoed text
	 */
    public java.lang.String echo(java.lang.String text) throws java.rmi.RemoteException {
		System.out.println("echo: " + text);
		displaySubject();
		
		return text;
    }

    /**
	 * Converts text to upper case.
	 * 
	 * @param text the text to convert
	 * @return the converted text
	 */
    public java.lang.String toUpperCase(java.lang.String text) throws java.rmi.RemoteException {
		System.out.println("toUpperCase: " + text);
		displaySubject();

		return text.toUpperCase();
    }

	/**
	 * Converts text to lower case.
	 * 
	 * @param text the text to convert
	 * @return the converted text
	 */
    public java.lang.String toLowerCase(java.lang.String text) throws java.rmi.RemoteException {
		System.out.println("toLowerCase: " + text);
		displaySubject();

		return text.toLowerCase();
    }

	/**
	 * Returns information about the identity under which the web service is running.
	 * 
	 * @return the identity information
	 */
	public java.lang.String whoAmI() throws java.rmi.RemoteException {
		System.out.println("whoAmI");

		StringBuffer stringBuffer = new StringBuffer();

		displaySubject(stringBuffer);

		String displayText = stringBuffer.toString();

		System.out.print(displayText);

		return displayText;
	}
}
